@@ -175,6 +175,26 @@ int		SG_Get_History_Ignore_Lists		(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool		SG_Data_Object_Delete		(CSG_Data_Object *pObject)
+{
+	if( pObject && !pObject->Get_Managed() )
+	{
+		delete(pObject);
+
+		return( true );
+	}
+
+	return( false );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -184,34 +204,77 @@ int		SG_Get_History_Ignore_Lists		(void)
 //---------------------------------------------------------
 CSG_Data_Object::CSG_Data_Object(void)
 {
+	static int RefCount = 0;
+
+	#pragma omp critical
+	{
+		m_RefID       = ++RefCount;
+	}
+
+	m_pOwner          = NULL;
+
+	m_File_bNative    = false;
+	m_File_Type       = 0;
+
+	m_NoData_Value[0] = -99999.;
+	m_NoData_Value[1] = -99999.;
+
+	m_Max_Samples     = gSG_DataObject_Max_Samples;
+	m_bModified       = true;
+	m_bUpdate         = false;
+
 	m_MetaData.Set_Name("SAGA_METADATA");
 
-	m_pMD_Database		= m_MetaData.Add_Child(SG_META_DATABASE);
-	m_pMD_Source		= m_MetaData.Add_Child(SG_META_SOURCE  );
-	m_pMD_History		= m_MetaData.Add_Child(SG_META_HISTORY );
+	m_pMD_Database    = m_MetaData.Add_Child(SG_META_DATABASE);
+	m_pMD_Source      = m_MetaData.Add_Child(SG_META_SOURCE  );
+	m_pMD_History     = m_MetaData.Add_Child(SG_META_HISTORY );
 
-	//-----------------------------------------------------
-	m_File_bNative		= false;
-	m_File_Type			= 0;
-	m_bModified			= true;
+	#ifdef WITH_LIFETIME_TRACKER
+	#pragma omp critical
+	{
+		++m_Track_nObjects;
 
-	m_NoData_Value[0]	= -99999.;
-	m_NoData_Value[1]	= -99999.;
-
-	m_Max_Samples		= gSG_DataObject_Max_Samples;
-
-	m_Name				.Clear();
-	m_Description		.Clear();
-
-	m_bUpdate			= false;
-
-	m_pOwner			= NULL;
+		if( m_Track )
+		{
+			SG_UI_Console_Print_StdOut(CSG_String::Format("data object (refid=%04d) constructed, new object count is %d", m_RefID, m_Track_nObjects - m_Track_Offset));
+		}
+	}
+	#endif // WITH_LIFETIME_TRACKER
 }
 
 //---------------------------------------------------------
 CSG_Data_Object::~CSG_Data_Object(void)
 {
 	Destroy();
+
+	#ifdef WITH_LIFETIME_TRACKER
+	#pragma omp critical
+	{
+		--m_Track_nObjects;
+
+		if( m_Track )
+		{
+			SG_UI_Console_Print_StdOut(CSG_String::Format("data object (refid=%04d) destructed,  new object count is %d", m_RefID, m_Track_nObjects - m_Track_Offset));
+		}
+	}
+	#endif // WITH_LIFETIME_TRACKER
+}
+
+//---------------------------------------------------------
+bool CSG_Data_Object::m_Track = false; int CSG_Data_Object::m_Track_nObjects = 0; int CSG_Data_Object::m_Track_Offset = 0;
+
+//---------------------------------------------------------
+void CSG_Data_Object::Track(bool Track, bool Offset)
+{
+	#ifdef WITH_LIFETIME_TRACKER
+	m_Track = Track; m_Track_Offset = Offset ? m_Track_nObjects : 0;
+
+	SG_UI_Console_Print_StdOut(CSG_String::Format("data object construction/destruction tracker, state=%s, offset=%s, current object count is %d",
+		m_Track ? SG_T("ON") : SG_T("OFF"), m_Track_Offset ? SG_T("ON") : SG_T("OFF"), m_Track_nObjects
+	));
+	#else
+	SG_UI_Console_Print_StdOut("CSG_Data_Object::Track() functionality (aka data object lifetime tracker) has not been built for this configuration!");
+	#endif // WITH_LIFETIME_TRACKER
 }
 
 //---------------------------------------------------------
@@ -232,24 +295,24 @@ bool CSG_Data_Object::Destroy(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
+void CSG_Data_Object::Set_Name(const char       *Name) { Set_Name(CSG_String(Name)); }
+void CSG_Data_Object::Set_Name(const wchar_t    *Name) { Set_Name(CSG_String(Name)); }
 void CSG_Data_Object::Set_Name(const CSG_String &Name)
 {
 	if( Name.is_Empty() )
 	{
-		m_Name	= _TL("Data");
+		m_Name = _TL("Data");
 	}
 	else
 	{
-		m_Name	= Name;
+		m_Name = Name;
 	}
 }
 
 //---------------------------------------------------------
 void CSG_Data_Object::Fmt_Name(const char *Format, ...)
 {
-	wxString	_s;
-
-	va_list	argptr;
+	wxString _s; va_list argptr;
 	
 #ifdef _SAGA_LINUX
 	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
@@ -266,9 +329,7 @@ void CSG_Data_Object::Fmt_Name(const char *Format, ...)
 //---------------------------------------------------------
 void CSG_Data_Object::Fmt_Name(const wchar_t *Format, ...)
 {
-	wxString	_s;
-
-	va_list	argptr;
+	wxString _s; va_list argptr;
 	
 #ifdef _SAGA_LINUX
 	// workaround as we only use wide characters
@@ -281,8 +342,6 @@ void CSG_Data_Object::Fmt_Name(const wchar_t *Format, ...)
 
 	va_end(argptr);
 
-	CSG_String	s(&_s);
-
 	Set_Name(CSG_String(&_s));
 }
 
@@ -295,7 +354,7 @@ const SG_Char * CSG_Data_Object::Get_Name(void) const
 //---------------------------------------------------------
 void CSG_Data_Object::Set_Description(const CSG_String &Description)
 {
-	m_Description	= Description;
+	m_Description = Description;
 }
 
 const SG_Char * CSG_Data_Object::Get_Description(void) const
@@ -733,6 +792,18 @@ bool CSG_Data_Object::Update(bool bForce)
 //---------------------------------------------------------
 bool CSG_Data_Object::Assign(CSG_Data_Object *pObject)
 {
+	if( pObject )// && pObject->is_Valid() )
+	{
+		Destroy();
+
+		m_Name = pObject->Get_Name();
+
+		Get_MetaData() = pObject->Get_MetaData();
+	//	Get_History () = pObject->Get_History ();
+
+		return( true );
+	}
+
 	return( false );
 }
 
