@@ -55,9 +55,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -208,9 +208,9 @@ TSG_Parameter_Type SG_Parameter_Type_Get_Type(const CSG_String &Identifier)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Node							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -222,9 +222,9 @@ CSG_Parameter_Node::CSG_Parameter_Node(CSG_Parameters *pOwner, CSG_Parameter *pP
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Bool							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -319,9 +319,9 @@ bool CSG_Parameter_Bool::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Value							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -421,9 +421,9 @@ bool CSG_Parameter_Value::_Assign(CSG_Parameter *pSource)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Int								 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -510,9 +510,9 @@ bool CSG_Parameter_Int::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Double							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -599,9 +599,9 @@ bool CSG_Parameter_Double::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Degree							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -625,9 +625,9 @@ void CSG_Parameter_Degree::_Set_String(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Date							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -737,9 +737,9 @@ bool CSG_Parameter_Date::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Range							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -882,9 +882,9 @@ bool CSG_Parameter_Range::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Choice							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1124,9 +1124,9 @@ bool CSG_Parameter_Choice::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Data Type						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1256,9 +1256,9 @@ bool CSG_Parameter_Data_Type::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Choices							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1438,9 +1438,9 @@ bool CSG_Parameter_Choices::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						String							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1507,9 +1507,9 @@ bool CSG_Parameter_String::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Text							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1521,9 +1521,9 @@ CSG_Parameter_Text::CSG_Parameter_Text(CSG_Parameters *pOwner, CSG_Parameter *pP
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						FilePath						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1618,9 +1618,9 @@ bool CSG_Parameter_File_Name::_Assign(CSG_Parameter *pSource)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Font							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1725,9 +1725,9 @@ bool CSG_Parameter_Font::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Color							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1778,9 +1778,9 @@ bool CSG_Parameter_Color::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Colors							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1866,9 +1866,9 @@ bool CSG_Parameter_Colors::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						FixedTable						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1976,9 +1976,9 @@ bool CSG_Parameter_Fixed_Table::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Grid_System						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2136,9 +2136,9 @@ bool CSG_Parameter_Grid_System::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					Table Field							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2314,9 +2314,9 @@ CSG_String CSG_Parameter_Table_Field::Get_Choices(const CSG_Table &Table, bool b
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					Table Fields						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2451,9 +2451,9 @@ bool CSG_Parameter_Table_Fields::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						DataObject						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2594,9 +2594,9 @@ bool CSG_Parameter_Data_Object::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					DataObject_Output					 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2661,9 +2661,9 @@ bool CSG_Parameter_Data_Object_Output::Set_DataObject_Type(TSG_Data_Object_Type
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Grid							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2682,12 +2682,7 @@ void CSG_Parameter_Grid::Set_Preferred_Type(TSG_Data_Type Type)
 //---------------------------------------------------------
 CSG_Grid_System * CSG_Parameter_Grid::Get_System(void)	const
 {
-	if( Get_Parent() && Get_Parent()->Get_Type() == PARAMETER_TYPE_Grid_System )
-	{
-		return( Get_Parent()->asGrid_System() );
-	}
-
-	return( NULL );
+	return( Get_Parent() ? Get_Parent()->asGrid_System() : NULL );
 }
 
 //---------------------------------------------------------
@@ -2802,9 +2797,9 @@ bool CSG_Parameter_Grid::_Assign(CSG_Parameter *pSource)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Grids							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2816,9 +2811,9 @@ CSG_Parameter_Grids::CSG_Parameter_Grids(CSG_Parameters *pOwner, CSG_Parameter *
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Table							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2857,9 +2852,9 @@ int CSG_Parameter_Table::_Set_Value(void *Value)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Shapes							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2918,9 +2913,9 @@ bool CSG_Parameter_Shapes::_Assign(CSG_Parameter *pSource)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						TIN								 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2959,9 +2954,9 @@ int CSG_Parameter_TIN::_Set_Value(void *Value)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						PointCloud						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -3006,9 +3001,9 @@ bool CSG_Parameter_PointCloud::_Assign(CSG_Parameter *pSource)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						List							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -3073,6 +3068,22 @@ bool CSG_Parameter_List::Del_Items(void)
 	return( false );
 }
 
+//---------------------------------------------------------
+int CSG_Parameter_List::_Set_Value(void *Value)
+{
+	if( Value == NULL && Get_Item_Count() > 0 && Del_Items() )
+	{
+		return( SG_PARAMETER_DATA_SET_CHANGED );
+	}
+
+	if( Value != NULL && Add_Item((CSG_Data_Object *)Value) )
+	{
+		return( SG_PARAMETER_DATA_SET_CHANGED );
+	}
+
+	return( SG_PARAMETER_DATA_SET_TRUE );
+}
+
 //---------------------------------------------------------
 void CSG_Parameter_List::_Set_String(void)
 {
@@ -3161,9 +3172,9 @@ bool CSG_Parameter_List::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Grid_List						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -3301,9 +3312,9 @@ bool CSG_Parameter_Grid_List::Update_Data(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Grids_List						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -3378,9 +3389,9 @@ bool CSG_Parameter_Grids_List::Add_Item(CSG_Data_Object *pObject)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Table_List						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 
@@ -3393,9 +3404,9 @@ CSG_Parameter_Table_List::CSG_Parameter_Table_List(CSG_Parameters *pOwner, CSG_P
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Shapes_List						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 
@@ -3422,9 +3433,9 @@ bool CSG_Parameter_Shapes_List::_Assign(CSG_Parameter *pSource)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						TIN_List						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 
@@ -3437,9 +3448,9 @@ CSG_Parameter_TIN_List::CSG_Parameter_TIN_List(CSG_Parameters *pOwner, CSG_Param
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					PointCloud_List						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 
@@ -3452,9 +3463,9 @@ CSG_Parameter_PointCloud_List::CSG_Parameter_PointCloud_List(CSG_Parameters *pOw
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Parameters						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -3514,9 +3525,9 @@ bool CSG_Parameter_Parameters::_Serialize(CSG_MetaData &Entry, bool bSave)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
