@@ -55,9 +55,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -70,9 +70,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -85,9 +85,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -112,9 +112,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -168,9 +168,9 @@ TSG_Parameter_Type;
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -180,9 +180,9 @@ SAGA_API_DLL_EXPORT TSG_Parameter_Type	SG_Parameter_Type_Get_Type			(const CSG_S
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -197,9 +197,9 @@ SAGA_API_DLL_EXPORT TSG_Parameter_Type	SG_Parameter_Type_Get_Type			(const CSG_S
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					CSG_Parameter						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -217,6 +217,7 @@ public:	///////////////////////////////////////////////////
 
 	const SG_Char *					Get_Identifier			(void)	const;
 	bool							Cmp_Identifier			(const CSG_String &Identifier)	const;
+	CSG_String						Get_CmdID				(void)	const;
 	bool							Set_Name				(const CSG_String &Name);
 	const SG_Char *					Get_Name				(void)	const;
 	const SG_Char *					Get_Description			(void)	const;
@@ -1099,6 +1100,8 @@ public:
 	int							Get_Count				(void)	const	{	return( m_nFields );	}
 	int							Get_Index				(int i)	const	{	return( i >= 0 && i < m_nFields ? m_Fields[i] : -1 );	}
 
+	int							operator []				(int i)	const	{	return( i >= 0 && i < m_nFields ? m_Fields[i] : -1 );	}
+
 	CSG_Table *					Get_Table				(void)	const;
 
 
@@ -1375,6 +1378,8 @@ protected:
 	CSG_Parameter_List(CSG_Parameters *pOwner, CSG_Parameter *pParent, const CSG_String &ID, const CSG_String &Name, const CSG_String &Description, int Constraint);
 
 
+	virtual int					_Set_Value				(void *Value);
+
 	virtual void				_Set_String				(void);
 
 	virtual int					_asInt					(void)	const;
@@ -1589,9 +1594,41 @@ protected:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//				Grid Target Selector					 //
-//														 //
+//                                                       //
+//         Coordinate System Reference Picker            //
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Parameters_CRSPicker
+{
+public:
+	CSG_Parameters_CRSPicker(void);
+
+	bool						Create					(CSG_Parameters &Parameters, const CSG_String &ParentID = "");
+
+	bool						Activate_GUI			(bool bReset = false);
+	bool						Deactivate_GUI			(void);
+
+	static bool					On_Parameter_Changed	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+	static bool					On_Parameters_Enable	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+
+	bool						Get_CRS					(CSG_Projection &Projection, bool bMessage = false)	const;
+
+
+private:
+
+	CSG_Parameters				*m_pParameters { NULL };
+
+	class CSG_Tool				*m_pCRS { NULL };
+
+};
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
+//                Grid Target Selector                   //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1618,7 +1655,8 @@ public:
 
 	bool						Set_User_Defined_ZLevels	(CSG_Parameters *pParameters, double zMin, double zMax, int nLevels, int Rounding = 2);
 
-	CSG_Grid_System				Get_System					(void);
+	CSG_Grid_System				Get_System					(void)                        const;
+	CSG_Grid_System				Get_System					(CSG_Parameters *pParameters) const;
 
 	CSG_Grid *					Get_Grid					(const CSG_String &ID, TSG_Data_Type Type = SG_DATATYPE_Float);
 	CSG_Grid *					Get_Grid					(                      TSG_Data_Type Type = SG_DATATYPE_Float);
@@ -1637,9 +1675,9 @@ private:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					CSG_Parameters						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1670,10 +1708,14 @@ public:
 	void *						Get_Owner				(void)	const	{	return( m_pOwner      );	}
 
 	class CSG_Tool *			Get_Tool				(void)	const	{	return( m_pTool       );	}
+	void						Set_Tool				(class CSG_Tool *pTool);
 
 	class CSG_Data_Manager *	Get_Manager				(void)	const	{	return( m_pManager    );	}
 	void						Set_Manager				(class CSG_Data_Manager *pManager);
 
+	bool						Push					(class CSG_Data_Manager *pManager = NULL, bool bRestoreDefaults = true);
+	bool						Pop						(void);
+
 	bool						Use_Grid_System			(void);
 
 	bool						has_GUI					(void)	const;
@@ -1705,8 +1747,11 @@ public:
 	//-----------------------------------------------------
 	CSG_Parameters &			operator =				(const CSG_Parameters &Parameters)	{	Create(Parameters);	return( *this );	}
 
-	CSG_Parameter *				Get_Parameter			(int i               )	const	{	return( i >= 0 && i < m_nParameters ? m_Parameters[i] : NULL );	}
-	CSG_Parameter *				Get_Parameter			(const CSG_String &ID)	const;
+	CSG_Parameter *				Get_Parameter			(int i                                        )	const { return( i >= 0 && i < m_nParameters ? m_Parameters[i] : NULL ); }
+	CSG_Parameter *				Get_Parameter			(int i               , bool MsgOnError        )	const;
+	CSG_Parameter *				Get_Parameter			(const CSG_String &ID, bool MsgOnError = false)	const;
+	CSG_Parameter *				Get_Parameter			(const char       *ID, bool MsgOnError = false)	const;
+	CSG_Parameter *				Get_Parameter			(const wchar_t    *ID, bool MsgOnError = false)	const;
 
 	CSG_Parameter *				operator()				(int i               )	const	{	return(  Get_Parameter(i ) );	}
 	CSG_Parameter *				operator()				(const CSG_String &ID)	const	{	return(  Get_Parameter(ID) );	}
@@ -1867,6 +1912,8 @@ private:
 
 	CSG_Parameter				**m_Parameters, *m_pGrid_System;
 
+	CSG_Parameters				*m_pStack;
+
 	TSG_PFNC_Parameter_Changed	m_Callback;
 
 
@@ -1976,9 +2023,9 @@ public:	// DEPRECATED BEGIN
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
