@@ -89,9 +89,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -99,9 +99,9 @@ CWKSP_Data_Manager	*g_pData	= NULL;
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -356,7 +356,7 @@ CWKSP_Data_Manager::~CWKSP_Data_Manager(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -460,7 +460,7 @@ bool CWKSP_Data_Manager::Finalise(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -549,7 +549,7 @@ wxMenu * CWKSP_Data_Manager::Get_Menu(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -691,7 +691,7 @@ bool CWKSP_Data_Manager::On_Command_UI(wxUpdateUIEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -745,7 +745,7 @@ void CWKSP_Data_Manager::Parameters_Changed(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -797,37 +797,37 @@ int CWKSP_Data_Manager::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Pa
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CWKSP_Base_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
+CWKSP_Data_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
 {
-	CSG_Data_Object	*pObject	= NULL;
+	CSG_Data_Object *pObject = NULL;
 
 //	SG_Get_Data_Manager().Add(File, DataType);
 
 	switch( DataType )
 	{
-	case SG_DATAOBJECT_TYPE_Table     :	pObject	= SG_Create_Table     (&File);	break;
-	case SG_DATAOBJECT_TYPE_Shapes    :	pObject	= SG_Create_Shapes    (&File);	break;
-	case SG_DATAOBJECT_TYPE_TIN       :	pObject	= SG_Create_TIN       (&File);	break;
-	case SG_DATAOBJECT_TYPE_PointCloud:	pObject	= SG_Create_PointCloud(&File);	break;
-	case SG_DATAOBJECT_TYPE_Grid      :	pObject	= SG_Create_Grid      (&File);	break;
-	case SG_DATAOBJECT_TYPE_Grids     :	pObject	= SG_Create_Grids     (&File);	break;
+	case SG_DATAOBJECT_TYPE_Table     : pObject = SG_Create_Table     (&File); break;
+	case SG_DATAOBJECT_TYPE_Shapes    : pObject = SG_Create_Shapes    (&File); break;
+	case SG_DATAOBJECT_TYPE_TIN       : pObject = SG_Create_TIN       (&File); break;
+	case SG_DATAOBJECT_TYPE_PointCloud: pObject = SG_Create_PointCloud(&File); break;
+	case SG_DATAOBJECT_TYPE_Grid      : pObject = SG_Create_Grid      (&File); break;
+	case SG_DATAOBJECT_TYPE_Grids     : pObject = SG_Create_Grids     (&File); break;
 	}
 
 	if( pObject )
 	{
-		CWKSP_Data_Item	*pItem;
+		CWKSP_Data_Item *pItem = Add(pObject);
 
-		if( pObject->is_Valid() && (pItem = Add(pObject)) != NULL )
+		if( pItem )
 		{
 			m_pMenu_Files->Recent_Add(pObject->Get_ObjectType(), File);
 
 			SG_Get_Data_Manager().Add(pObject);
 
-			return( (CWKSP_Base_Item *)pItem );
+			return( pItem );
 		}
 
 		delete(pObject);
@@ -841,7 +841,9 @@ CWKSP_Base_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::Open(const wxString &File)
 {
-	if( SG_File_Cmp_Extension(&File, "txt") && File.Right(8).CmpNoCase("_MTL.txt") == 0 )
+	if( (SG_File_Cmp_Extension(&File, "txt" ) && !File.Right(8).CmpNoCase("_MTL.txt" ))
+	||  (SG_File_Cmp_Extension(&File, "xml" ) && !File.Right(8).CmpNoCase("_MTL.xml" ))
+	||  (SG_File_Cmp_Extension(&File, "json") && !File.Right(9).CmpNoCase("_MTL.json")) )
 	{
 		CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool("imagery_tools", 14); // Import Landsat Scene
 
@@ -906,12 +908,8 @@ bool CWKSP_Data_Manager::Open(const wxString &File)
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::Open(int DataType)
 {
-	bool	bResult	= false;
+	int ID;
 
-	int				ID;
-	wxArrayString	Files;
-
-	//-----------------------------------------------------
 	switch( DataType )
 	{
 	case SG_DATAOBJECT_TYPE_Table     : ID = ID_DLG_TABLE_OPEN     ; break;
@@ -924,6 +922,8 @@ bool CWKSP_Data_Manager::Open(int DataType)
 	}
 
 	//-----------------------------------------------------
+	bool bResult = false; wxArrayString Files;
+
 	if( DLG_Open(Files, ID) )
 	{
 		MSG_General_Add_Line();
@@ -932,12 +932,12 @@ bool CWKSP_Data_Manager::Open(int DataType)
 		{
 			if( Open(Files[i], DataType) )
 			{
-				bResult	= true;
+				bResult = true;
 			}
 		}
 	}
 
-	return( true );
+	return( bResult );
 }
 
 //---------------------------------------------------------
@@ -948,7 +948,7 @@ bool CWKSP_Data_Manager::Open_CMD(int Cmd_ID)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1023,7 +1023,7 @@ bool CWKSP_Data_Manager::Open_Browser(wxArrayString &Projects, const wxString &D
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1105,8 +1105,6 @@ int CWKSP_Data_Manager::_Modified_Changed(CSG_Parameter *pParameter, int Flags)
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::_Modified_Get(CSG_Parameters *pParameters, CWKSP_Base_Item *pItem, const wxString &Directory, bool bSelections)
 {
-	int		i;
-
 	if( pItem && pParameters )
 	{
 		switch( pItem->Get_Type() )
@@ -1123,7 +1121,7 @@ bool CWKSP_Data_Manager::_Modified_Get(CSG_Parameters *pParameters, CWKSP_Base_I
 		case WKSP_ITEM_PointCloud_Manager:
 		case WKSP_ITEM_Grid_Manager      :
 		case WKSP_ITEM_Grid_System       :
-			for(i=0; i<((CWKSP_Base_Manager *)pItem)->Get_Count(); i++)
+			for(int i=0; i<((CWKSP_Base_Manager *)pItem)->Get_Count(); i++)
 			{
 				_Modified_Get(pParameters, ((CWKSP_Base_Manager *)pItem)->Get_Item(i), Directory, bSelections && !pItem->is_Selected());
 			}
@@ -1162,7 +1160,7 @@ bool CWKSP_Data_Manager::_Modified_Get(CSG_Parameters *pParameters, CWKSP_Base_I
 	}
 
 	//-----------------------------------------------------
-	wxString	Filter, Extension;
+	wxString Filter, Extension;
 
 	switch( pItem->Get_Type() )
 	{
@@ -1172,18 +1170,20 @@ bool CWKSP_Data_Manager::_Modified_Get(CSG_Parameters *pParameters, CWKSP_Base_I
 	case WKSP_ITEM_PointCloud: Filter = DLG_Get_FILE_Filter(ID_DLG_POINTCLOUD_SAVE); Extension = "sg-pts-z"; break;
 	case WKSP_ITEM_Grid      : Filter = DLG_Get_FILE_Filter(ID_DLG_GRID_SAVE      ); Extension = SG_Grid_Get_File_Extension_Default().c_str(); break;
 	case WKSP_ITEM_Grids     : Filter = DLG_Get_FILE_Filter(ID_DLG_GRIDS_SAVE     ); Extension = "sg-gds-z"; break;
-	default:	return( false );
+	default: return( false );
 	}
 
 	//-----------------------------------------------------
-	wxFileName	Path(pObject->Get_File_Name());
+	wxFileName Path(pObject->Get_File_Name());
 
 	if( !Path.FileExists() )
 	{
-		wxString	Name(pObject->Get_Name());
+		wxString Name(pObject->Get_Name());
 
-		Name.Replace(".", "-");
-		Name.Replace(":", "-");
+		Name.Replace("." , "_");
+		Name.Replace(":" , "_");
+		Name.Replace("/" , "_");
+		Name.Replace("\\", "_");
 
 		Path.SetPath(Directory);
 		Path.SetExt (Extension);
@@ -1199,15 +1199,15 @@ bool CWKSP_Data_Manager::_Modified_Get(CSG_Parameters *pParameters, CWKSP_Base_I
 				Path.SetName(Name + wxString::Format("_%d", i));
 			}
 
-			bOkay	= !Path.FileExists();
+			bOkay = !Path.FileExists();
 
 			for(int j=0; bOkay && j<pParameters->Get_Count(); j++)
 			{
-				CSG_Parameter	*p	= pParameters->Get_Parameter(j);
+				CSG_Parameter &P = *pParameters->Get_Parameter(j);
 
-				if( p->Get_Type() == PARAMETER_TYPE_FilePath && Path == p->asString() )
+				if( P.Get_Type() == PARAMETER_TYPE_FilePath && Path == P.asString() )
 				{
-					bOkay	= false;
+					bOkay = false;
 				}
 			}
 		}
@@ -1306,7 +1306,7 @@ bool CWKSP_Data_Manager::Close(bool bSilent)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1341,13 +1341,68 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Get(CSG_Data_Object *pObject)
 		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Data_Item *)m_pShapes     ->Get_Data((CSG_Shapes     *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Data_Item *)m_pGrids      ->Get_Data((CSG_Grid       *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Data_Item *)m_pGrids      ->Get_Data((CSG_Grids      *)pObject) );
-		default:	break;
+		default: break;
 		}
 	}
 
 	return( NULL );
 }
 
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Get_byFile(const wxString &File)
+{
+	return( Get(SG_Get_Data_Manager().Find(&File, false)) );
+}
+
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Get_byID(const wxString &ID)
+{
+	CWKSP_Data_Item *pItem;
+
+	if( m_pTables      && (pItem = (CWKSP_Data_Item *)m_pTables     ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pTINs        && (pItem = (CWKSP_Data_Item *)m_pTINs       ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pPointClouds && (pItem = (CWKSP_Data_Item *)m_pPointClouds->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pShapes      && (pItem = (CWKSP_Data_Item *)m_pShapes     ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+	if( m_pGrids       && (pItem = (CWKSP_Data_Item *)m_pGrids      ->Get_Item_byID(ID)) != NULL ) { return( pItem ); }
+
+	return( NULL );
+}
+
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Get_byID_or_File(const SG_Char *ID, const SG_Char *File, const SG_Char *Root)
+{
+	CWKSP_Data_Item *pItem = NULL;
+
+	if( ID )
+	{
+		pItem = Get_byID(ID);
+	}
+
+	if( File )
+	{
+		wxString _File(File);
+
+		if( _File.Find("PGSQL") != 0 && Root && SG_Dir_Exists(Root) )
+		{
+			_File = Get_FilePath_Absolute(Root, _File);
+		}
+
+		if( pItem )
+		{
+			if( _File.CmpNoCase(pItem->Get_Object()->Get_File_Name(false)) != 0 )
+			{
+				pItem = NULL;
+			}
+		}
+		else
+		{
+			pItem = Get_byFile(_File);
+		}
+	}
+
+	return( pItem );
+}
+
 //---------------------------------------------------------
 CWKSP_Data_Item * CWKSP_Data_Manager::Add(CSG_Data_Object *pObject)
 {
@@ -1361,7 +1416,7 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Add(CSG_Data_Object *pObject)
 		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Data_Item *)m_pShapes     ->Add_Data((CSG_Shapes     *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grid       *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grids      *)pObject) );
-		default:	break;
+		default: break;
 		}
 	}
 
@@ -1380,7 +1435,7 @@ CWKSP_Layer * CWKSP_Data_Manager::Get_Layer(CSG_Data_Object *pObject)
 		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Layer *)m_pShapes     ->Get_Data((CSG_Shapes     *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grid       *)pObject) );
 		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grids      *)pObject) );
-		default:	break;
+		default: break;
 		}
 	}
 
@@ -1404,7 +1459,7 @@ void CWKSP_Data_Manager::Del_Manager(CWKSP_Base_Item *pItem)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1516,7 +1571,7 @@ bool CWKSP_Data_Manager::Show_Diagram(CSG_Table *pTable, CSG_Parameters *pParame
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1623,7 +1678,8 @@ bool CWKSP_Data_Manager::MultiSelect_Check(void)
 							case PARAMETER_TYPE_Degree     :
 							case PARAMETER_TYPE_Color      :
 							case PARAMETER_TYPE_Table_Field:
-							case PARAMETER_TYPE_Choice     :	Parameter.Set_Value(0.0);	break;
+							case PARAMETER_TYPE_Data_Type  :
+							case PARAMETER_TYPE_Choice     :	Parameter.Set_Value(0.);	break;
 
 							case PARAMETER_TYPE_Choices    :
 							case PARAMETER_TYPE_String     :
@@ -1699,7 +1755,7 @@ bool CWKSP_Data_Manager::MultiSelect_Update(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1765,9 +1821,9 @@ CSG_Parameters * CWKSP_Data_Manager::Get_Settings_Dialog(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
