@@ -221,9 +221,9 @@ wxString CWKSP_Map::Get_Description(void)
 	wxString s;
 
 	//-----------------------------------------------------
-	s	+= wxString::Format("<h4>%s</h4>", _TL("Map"));
+	s += wxString::Format("<h4>%s</h4>", _TL("Map"));
 
-	s	+= "<table border=\"0\">";
+	s += "<table border=\"0\">";
 
 	DESC_ADD_STR(_TL("Name"             ), m_Name.c_str());
 	DESC_ADD_INT(_TL("Layers"           ), Get_Count());
@@ -238,7 +238,7 @@ wxString CWKSP_Map::Get_Description(void)
 	DESC_ADD_STR(_TL("North"            ), SG_Get_String(Get_Extent().Get_YMax  (), n).c_str());
 	DESC_ADD_STR(_TL("South-North"      ), SG_Get_String(Get_Extent().Get_YRange(), n).c_str());
 
-	s	+= "</table>";
+	s += "</table>";
 
 	//-----------------------------------------------------
 	return( s );
@@ -662,19 +662,22 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 
 				if( pObject && pObject->Get_File_Name(false) && *pObject->Get_File_Name(false) )
 				{
-					wxString FileName(pObject->Get_File_Name(false));
+					wxString FileName(pObject->Get_File_Name(false)); CSG_MetaData *pEntry = NULL;
 
 					if( FileName.Find("PGSQL") == 0 )
 					{
-						pLayer->Save_Settings(
-							Layers.Add_Child("FILE", &FileName)
-						);
+						pEntry = Layers.Add_Child("FILE", &FileName);
 					}
 					else if( wxFileExists(FileName) )
 					{
-						pLayer->Save_Settings(
-							Layers.Add_Child("FILE", SG_File_Get_Path_Relative(&ProjectDir, &FileName))
-						);
+						pEntry = Layers.Add_Child("FILE", SG_File_Get_Path_Relative(&ProjectDir, &FileName));
+					}
+
+					if( pEntry )
+					{
+						pEntry->Add_Property("dataset_id", pLayer->Get_Layer()->Get_Unique_ID().wc_str());
+
+						pLayer->Save_Settings(pEntry);
 					}
 				}
 				break; }
@@ -717,14 +720,7 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 
 			if( Layer.Cmp_Name("FILE") )
 			{
-				wxString FileName(Layer.Get_Content().w_str());
-
-				if( FileName.Find("PGSQL") != 0 )
-				{
-					FileName = Get_FilePath_Absolute(ProjectDir, FileName);
-				}
-
-				CWKSP_Base_Item *pItem = g_pData->Get(SG_Get_Data_Manager().Find(&FileName, false));
+				CWKSP_Data_Item *pItem = g_pData->Get_byID_or_File(Layer.Get_Property("dataset_id"), Layer.Get_Content(), ProjectDir);
 
 				if(	pItem &&
 				(   pItem->Get_Type() == WKSP_ITEM_Grid
@@ -745,15 +741,8 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 			}
 			else if( Layer.Cmp_Name("PARAMETERS") )
 			{
-				if( Layer.Cmp_Property("name", "GRATICULE") )
-				{
-					Add_Graticule(&Layer);
-				}
-
-				if( Layer.Cmp_Property("name", "BASEMAP") )
-				{
-					Add_BaseMap  (&Layer);
-				}
+				if( Layer.Cmp_Property("name", "GRATICULE") ) { Add_Graticule(&Layer); }
+				if( Layer.Cmp_Property("name", "BASEMAP"  ) ) { Add_BaseMap  (&Layer); }
 			}
 		}
 
@@ -792,17 +781,32 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 //---------------------------------------------------------
 bool CWKSP_Map::Update(CWKSP_Layer *pLayer, bool bMapOnly)
 {
-	bool	bRefresh	= false;
+	if( !m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay() )
+	{
+		m_Projection.Create(pLayer->Get_Object()->Get_Projection());
+	}
+
+	bool bRefresh = false;
 
 	for(int i=0; i<Get_Count(); i++)
 	{
-		if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer && ((CWKSP_Map_Layer *)Get_Item(i))->Get_Layer()->Update(pLayer) )
+		if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer )
 		{
-			bRefresh	= true;
-
-			if( !bMapOnly )
+			CWKSP_Map_Layer *pMapLayer = (CWKSP_Map_Layer *)Get_Item(i);
+			
+			if( pMapLayer->Get_Layer()->Update(pLayer) )
 			{
-				Get_Item(i)->Parameters_Changed();
+				bRefresh = true;
+
+				if( pMapLayer->Get_Layer() == pLayer )
+				{
+					pMapLayer->Fit_Colors(Get_Extent());
+				}
+
+				if( !bMapOnly )
+				{
+					pMapLayer->Parameters_Changed();
+				}
 			}
 		}
 	}
@@ -853,7 +857,7 @@ CWKSP_Map_Layer * CWKSP_Map::Get_Map_Layer(CWKSP_Layer *pLayer)
 //---------------------------------------------------------
 CWKSP_Map_Layer * CWKSP_Map::Get_Map_Layer_Active(bool bEditable)
 {
-	CWKSP_Map_Layer	*pLayer	= Get_Map_Layer(Get_Active_Layer());
+	CWKSP_Map_Layer *pLayer = Get_Map_Layer(Get_Active_Layer());
 
 	return( pLayer && !(bEditable && pLayer->is_Projecting()) ? pLayer : NULL );
 }
@@ -869,27 +873,27 @@ CWKSP_Map_Layer * CWKSP_Map::Add_Layer(CWKSP_Layer *pLayer)
 	}
 
 	//-----------------------------------------------------
-	bool	bProject	= false;
+	bool bProject = false;
 
 	if( m_Parameters("CRS_CHECK")->asBool()
 	&&  m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay()
 	&&  m_Projection.is_Equal(    pLayer->Get_Object()->Get_Projection()) == false )
 	{
-		wxString	s;
+		wxString s;
 
-		s	+= _TL("The coordinate system used by the layer is not identical with the one of the map!");
-		s	+= "\n";
-		s	+= wxString::Format("\n%s:\n  [%s]", _TL("Map"  ),                         m_Projection  .Get_Proj4().c_str());
-		s	+= wxString::Format("\n%s:\n  [%s]", _TL("Layer"), pLayer->Get_Object()->Get_Projection().Get_Proj4().c_str());
-		s	+= "\n\n";
-		s	+= _TL("Do you want to activate on-the-fly projection for this layer in the map?");
-		s	+= "\n";
-		s	+= _TL("(Press cancel if you decide not to add the layer at all!)");
+		s += _TL("The coordinate system used by the layer is not identical with the one of the map!");
+		s += "\n";
+		s += wxString::Format("\n%s:\n  [%s]", _TL("Map"  ),                         m_Projection  .Get_Proj4().c_str());
+		s += wxString::Format("\n%s:\n  [%s]", _TL("Layer"), pLayer->Get_Object()->Get_Projection().Get_Proj4().c_str());
+		s += "\n\n";
+		s += _TL("Do you want to activate on-the-fly projection for this layer in the map?");
+		s += "\n";
+		s += _TL("(Press cancel if you decide not to add the layer at all!)");
 
 		switch( DLG_Message_YesNoCancel(s, _TL("Add Layer to Map")) )
 		{
 		case  0: // yes
-			bProject	= true;
+			bProject = true;
 			break;
 
 		case  1: // no
@@ -902,11 +906,11 @@ CWKSP_Map_Layer * CWKSP_Map::Add_Layer(CWKSP_Layer *pLayer)
 
 	if( !m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay() )
 	{
-		m_Projection	= pLayer->Get_Object()->Get_Projection();
+		m_Projection = pLayer->Get_Object()->Get_Projection();
 	}
 
 	//-----------------------------------------------------
-	CWKSP_Map_Layer	*pMapLayer	= new CWKSP_Map_Layer(pLayer);
+	CWKSP_Map_Layer *pMapLayer = new CWKSP_Map_Layer(pLayer);
 
 	Add_Item(pMapLayer);
 
@@ -1147,11 +1151,11 @@ bool CWKSP_Map::Set_Extent_Full(void)
 //---------------------------------------------------------
 bool CWKSP_Map::Set_Extent_Active(bool bPan)
 {
-	CWKSP_Layer	*pLayer	= Get_Active_Layer();
+	CWKSP_Layer *pLayer = Get_Active_Layer();
 
 	if( pLayer )
 	{
-		CWKSP_Map_Layer	*pMapLayer	= Get_Map_Layer(pLayer);
+		CWKSP_Map_Layer *pMapLayer = Get_Map_Layer(pLayer);
 
 		if( pMapLayer )
 		{
@@ -1167,11 +1171,11 @@ bool CWKSP_Map::Set_Extent_Active(bool bPan)
 //---------------------------------------------------------
 bool CWKSP_Map::Set_Extent_Selection(bool bPan)
 {
-	CWKSP_Layer	*pLayer	= Get_Active_Layer();
+	CWKSP_Layer *pLayer = Get_Active_Layer();
 
 	if( pLayer )
 	{
-		CWKSP_Map_Layer	*pMapLayer	= Get_Map_Layer(Get_Active_Layer());
+		CWKSP_Map_Layer *pMapLayer = Get_Map_Layer(Get_Active_Layer());
 
 		if( pMapLayer && !pMapLayer->do_Project() )
 		{
@@ -1356,13 +1360,15 @@ void CWKSP_Map::Set_CrossHair_Off(void)
 //---------------------------------------------------------
 void CWKSP_Map::Set_Projection(void)
 {
-	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true);	// CCRS_Picker
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true);	// CCRS_Picker
 
 	if(	pTool )
 	{
-		CSG_Parameters	P(*pTool->Get_Parameters());
+		CSG_Parameters P(*pTool->Get_Parameters());
 
-		P.Set_Parameter("CRS_PROJ4", m_Projection.Get_Proj4());
+		P.Set_Parameter("CRS_PROJ4"    , m_Projection.Get_Proj4       ());
+		P.Set_Parameter("CRS_EPSG"     , m_Projection.Get_Authority_ID());
+		P.Set_Parameter("CRS_EPSG_AUTH", m_Projection.Get_Authority   ());
 
 		P.Add_Bool("", "ONTHEFLY", _TL("On-The-Fly Projection"),
 			_TL("Turn on the on-the-fly projection for all layers in the map."),
@@ -1372,7 +1378,12 @@ void CWKSP_Map::Set_Projection(void)
 		if(	DLG_Parameters(&P) && pTool->Get_Parameters()->Assign_Values(&P)
 		&&  pTool->Set_Manager(NULL) && pTool->On_Before_Execution() && pTool->Execute() )
 		{
-			CSG_Projection	Projection(pTool->Get_Parameter("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4);
+			CSG_Projection Projection; CSG_Parameters &_P = *pTool->Get_Parameters();
+
+			if( _P["CRS_EPSG"].asInt() < 0 || !Projection.Create(_P["CRS_EPSG"].asInt(), _P["CRS_EPSG_AUTH"].asString()) )
+			{
+				Projection.Create(_P["CRS_PROJ4"].asString(), SG_PROJ_FMT_Proj4);
+			}
 
 			if( P("ONTHEFLY")->asBool() )
 			{
@@ -1384,7 +1395,7 @@ void CWKSP_Map::Set_Projection(void)
 					}
 				}
 
-				CSG_Rect	r(Get_Extent());
+				CSG_Rect r(Get_Extent());
 
 				SG_Get_Projected(m_Projection, Projection, r);
 
@@ -1741,8 +1752,20 @@ bool CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 	CSG_Parameters P(_TL("Export Map to Google Earth"), SG_T(""), SG_T("SAVE_IMAGE"));
 	P.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
 
-	P.Add_FilePath("", "FILE"    , _TL("File"    ), _TL(""), CSG_String::Format("%s|*.kmz|%s|*.*", _TL("KMZ Files"), _TL("All Files")), NULL, true);
-	P.Add_Bool    ("", "LOAD"    , _TL("Load"    ), _TL(""), true);
+	P.Add_FilePath("",
+		"FILE", _TL("File"),
+		_TL("Omitting the target file will create it as temporary file."),
+		CSG_String::Format("%s|*.kmz|%s|*.*",
+			_TL("KMZ Files"),
+			_TL("All Files")
+		), NULL, true
+	);
+
+	P.Add_Bool("",
+		"OPEN", _TL("Open"),
+		_TL("Try to open KMZ file after creation with the associated application (e.g. \'Google Earth\')."),
+		true
+	);
 
 	CSG_Parameters_Grid_Target Target;
 	Target.Create(&P, true, "", "TARGET_");
@@ -1785,9 +1808,10 @@ bool CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 	Map.Set_NoData_Value(SG_GET_RGB(MASK_R, MASK_G, MASK_B));
 	Map.Get_Projection().Create(m_Projection);
 
-	for(int y=0, yy=Map.Get_NY()-1; y<Map.Get_NY(); y++, yy--)
+	#pragma omp parallel for
+	for(int y=0; y<Map.Get_NY(); y++)
 	{
-		for(int x=0; x<Map.Get_NX(); x++)
+		for(int x=0, yy=Map.Get_NY()-1-y; x<Map.Get_NX(); x++)
 		{
 			Map.Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
 		}
@@ -1799,11 +1823,11 @@ bool CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 	if(	pTool && pTool->Settings_Push()
 	&&  pTool->Set_Parameter("GRID"     , &Map)
 	&&  pTool->Set_Parameter("FILE"     , FileName.GetFullPath().wc_str())
-	&&  pTool->Set_Parameter("COLOURING", 4)	// rgb coded values
-	&&  pTool->Set_Parameter("OUTPUT"   , 2)	// kmz file
+	&&  pTool->Set_Parameter("COLOURING", 4) // rgb coded values
+	&&  pTool->Set_Parameter("FORMAT"   , 0) // 0 = png, 1 = jpg (default)
 	&&  pTool->Execute() )
 	{
-		if( P["LOAD"].asBool() )
+		if( P["OPEN"].asBool() )
 		{
 			Open_Application(FileName.GetFullPath());
 		}
